diff --git a/drivers/thunderbolt/retimer.c b/drivers/thunderbolt/retimer.c
index 8c29bd556ae04..8a402832449a3 100644
--- a/drivers/thunderbolt/retimer.c
+++ b/drivers/thunderbolt/retimer.c
@@ -16,6 +16,27 @@
 
 #define TB_MAX_RETIMER_INDEX	6
 
+static int connector_bind(struct device *dev, struct device *connector, void *data)
+{
+	int ret;
+
+	ret = sysfs_create_link(&dev->kobj, &connector->kobj, "connector");
+	if (ret)
+		return ret;
+
+	ret = sysfs_create_link(&connector->kobj, &dev->kobj, dev_name(dev));
+	if (ret)
+		sysfs_remove_link(&dev->kobj, "connector");
+
+	return ret;
+}
+
+static void connector_unbind(struct device *dev, struct device *connector, void *data)
+{
+	sysfs_remove_link(&connector->kobj, dev_name(dev));
+	sysfs_remove_link(&dev->kobj, "connector");
+}
+
 static int tb_retimer_nvm_read(void *priv, unsigned int offset, void *val,
 			       size_t bytes)
 {
@@ -389,6 +410,17 @@ static int tb_retimer_add(struct tb_port *port, u8 index, u32 auth_status)
 		return ret;
 	}
 
+	rt->connector_ops.bind = connector_bind;
+	rt->connector_ops.unbind = connector_unbind;
+
+	ret = component_add(&rt->dev, &rt->connector_ops);
+	if (ret) {
+		dev_warn(&rt->dev, "failed to add component\n");
+		tb_nvm_free(rt->nvm);
+		device_unregister(&rt->dev);
+		return ret;
+	}
+
 	dev_info(&rt->dev, "new retimer found, vendor=%#x device=%#x\n",
 		 rt->vendor, rt->device);
 
@@ -405,6 +437,7 @@ static int tb_retimer_add(struct tb_port *port, u8 index, u32 auth_status)
 static void tb_retimer_remove(struct tb_retimer *rt)
 {
 	dev_info(&rt->dev, "retimer disconnected\n");
+	component_del(&rt->dev, &rt->connector_ops);
 	tb_nvm_free(rt->nvm);
 	device_unregister(&rt->dev);
 }
diff --git a/drivers/thunderbolt/tb.h b/drivers/thunderbolt/tb.h
index b6fcd8d453243..dfa51bdbc3d83 100644
--- a/drivers/thunderbolt/tb.h
+++ b/drivers/thunderbolt/tb.h
@@ -9,6 +9,7 @@
 #ifndef TB_H_
 #define TB_H_
 
+#include <linux/component.h>
 #include <linux/nvmem-provider.h>
 #include <linux/pci.h>
 #include <linux/thunderbolt.h>
@@ -284,6 +285,7 @@ struct usb4_port {
 	struct tb_port *port;
 	bool can_offline;
 	bool offline;
+	struct component_ops connector_ops;
 };
 
 /**
@@ -306,6 +308,7 @@ struct tb_retimer {
 	struct tb_port *port;
 	struct tb_nvm *nvm;
 	u32 auth_status;
+	struct component_ops connector_ops;
 };
 
 /**
diff --git a/drivers/thunderbolt/usb4_port.c b/drivers/thunderbolt/usb4_port.c
index 29e2a4f9c9f5e..6df4247c8f7ac 100644
--- a/drivers/thunderbolt/usb4_port.c
+++ b/drivers/thunderbolt/usb4_port.c
@@ -10,6 +10,27 @@
 
 #include "tb.h"
 
+static int connector_bind(struct device *dev, struct device *connector, void *data)
+{
+	int ret;
+
+	ret = sysfs_create_link(&dev->kobj, &connector->kobj, "connector");
+	if (ret)
+		return ret;
+
+	ret = sysfs_create_link(&connector->kobj, &dev->kobj, dev_name(dev));
+	if (ret)
+		sysfs_remove_link(&dev->kobj, "connector");
+
+	return ret;
+}
+
+static void connector_unbind(struct device *dev, struct device *connector, void *data)
+{
+	sysfs_remove_link(&connector->kobj, dev_name(dev));
+	sysfs_remove_link(&dev->kobj, "connector");
+}
+
 static ssize_t link_show(struct device *dev, struct device_attribute *attr,
 			 char *buf)
 {
@@ -253,6 +274,16 @@ struct usb4_port *usb4_port_device_add(struct tb_port *port)
 	pm_runtime_mark_last_busy(&usb4->dev);
 	pm_runtime_use_autosuspend(&usb4->dev);
 
+	usb4->connector_ops.bind = connector_bind;
+	usb4->connector_ops.unbind = connector_unbind;
+
+	ret = component_add(&usb4->dev, &usb4->connector_ops);
+	if (ret) {
+		dev_warn(&usb4->dev, "failed to add component\n");
+		device_unregister(&usb4->dev);
+		return ERR_PTR(ret);
+	}
+
 	return usb4;
 }
 
@@ -265,6 +296,7 @@ struct usb4_port *usb4_port_device_add(struct tb_port *port)
  */
 void usb4_port_device_remove(struct usb4_port *usb4)
 {
+	component_del(&usb4->dev, &usb4->connector_ops);
 	device_unregister(&usb4->dev);
 }
 
